@@ -1,17 +1,19 @@
-/************************************************************
-*   mpgtx an mpeg toolbox                                      *
-*   by Laurent Alacoque <laureck@users.sourceforge.net>     *   
-*   (c) 2001                                                *
-*   You may copy, modify and redistribute this              *
-*   source file under the terms of the GNU Public License   *
-************************************************************/
+/*************************************************************
+ *   mpgtx an mpeg toolbox                                   *
+ *   by Laurent Alacoque <laureck@users.sourceforge.net>     *
+ *   (c) 2001                                                *
+ *   You may copy, modify and redistribute this              *
+ *   source file under the terms of the GNU Public License   *
+ ************************************************************/
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include "mpeg.hh"
 #include "mpegOut.hh"
 #include "chunkTab.hh"
-#include "id3command.hh"
+
+// declared in id3command.hh
+extern int ParseID3Command(int argc, char** argv, int argc_offset);
 
 // philipp
 bool desperate_mode = false;
@@ -37,7 +39,12 @@ int myargc;
 c_char mpgtx_stack[mpgtx_stack_count];
 int mpgtx_stack_current=0;
 
-
+/*!
+ * Callback for SIGSEGV, SIGBUS and SIGFPE.
+ * prints a formated bug report, warns user and opens the bug report file
+ *
+ * @param signum catched signal: SIGSEGV, SIGBUS or SIGFPE
+ */
 void CatchBug(int signum)
 {
 	//print a formated bug report and warn user
@@ -60,7 +67,7 @@ void CatchBug(int signum)
 		fprintf(stderr,"Please include the file \"bug.report\" and the program output in the bug report\n");
 	}
 
-	//start of bug report 
+	//start of bug report
 	//version
 	fprintf(bugreport,"%s version %s\n",progname,MPGTX_VERSION);
 
@@ -82,7 +89,7 @@ void CatchBug(int signum)
 	//print the calling stack
 	fprintf(bugreport,"stack : ");
 	for (int j=0; j <mpgtx_stack_count; j++){
-		if (mpgtx_stack[(mpgtx_stack_current+j)%mpgtx_stack_count]!=0) 
+		if (mpgtx_stack[(mpgtx_stack_current+j)%mpgtx_stack_count]!=0)
 			fprintf(bugreport,"[%s] ",mpgtx_stack[(mpgtx_stack_current+j)%mpgtx_stack_count]);
 	}
 	fprintf(bugreport,"\nstack current: %d\n",mpgtx_stack_current);
@@ -93,6 +100,11 @@ void CatchBug(int signum)
 	exit(1);
 }
 
+/*!
+ * Callback for SIGINT.
+ *
+ * @param signum unused
+ */
 void CatchBreak(int signum) {
 	fprintf(stderr,"\nHave a Nice Day\n");
 	exit(1);
@@ -104,9 +116,10 @@ void CatchBreak(int signum) {
 #include <unistd.h>
 #include <stdlib.h>
 
-	struct timeval tv;
-	struct timezone tz;
+struct timeval tv;
+struct timezone tz;
 double MainClockStart;
+
 double Clock_Now(){
 	double time_now;
 	gettimeofday(&tv,&tz);
@@ -118,7 +131,7 @@ double Clock_Now(){
 }
 
 void   AddTime(double timestart, int function){
-	functions_cummulated_time[function]+= Clock_Now()-timestart;	
+	functions_cummulated_time[function]+= Clock_Now()-timestart;
 }
 
 double functions_cummulated_time[N_FUNCTIONS];
@@ -179,11 +192,13 @@ void print_help(programname invocation){
 		case mpginfo:
 			printf("mpginfo  : print informations about mpeg files\n");
 			printf("Usage    : mpginfo file [file...]\n");
+			printf("Options  :\n");
+			printf("  -N              Do not check timeline (save some time).\n");
 			return;
 			break;
 		default :
 			printf("%s : manipulate mpeg files\n",progname);
-			break;	
+			break;
 	}
 
 	if (invocation != mpgdemux)
@@ -209,7 +224,8 @@ void print_help(programname invocation){
 		printf("                  Otherwise mpgtx will be dead slow!\n");
 		printf("  -f              force overwriting of files, don't ask for confirmation\n");
 		printf("  -P              keep headers of mpegs which sometimes is usefull\n");
-		printf("  -X              Try to search for a valid PACK header in the whole file\n");
+		printf("  -X              Try to search for a valid PACK header in the whole file.\n");
+		printf("                  Additionally option -N suppress output during search.\n");
 		printf("                  This may result in an endless loop if not successfull!\n");
 		printf("  -An             While joining or splitting, aspect ratio will be forced to:\n");
 		printf("                  n=1 -> 1:1, n=2 -> 4:3, n=3 -> 16:9, n=4 -> 2.21:1 \n");
@@ -221,7 +237,7 @@ void print_help(programname invocation){
 			printf("                  if FILE is - , standard output will be used\n");
 			printf("  --force         force joining of incompatible files\n");
 		}
-		printf("  --no-parachute  don't try to catch SIGSEGV (usefull for debugging)\n"); 
+		printf("  --no-parachute  don't try to catch SIGSEGV (usefull for debugging)\n");
 	}
 	if (invocation==mpgdemux) return;
 	printf("Ranges   :\n");
@@ -280,7 +296,7 @@ int main (int argc,char** argv){
 	init_cummulated_time();
 #endif
 
-	mpegOutFactory TheFac;	
+	mpegOutFactory TheFac;
 	chunkTab Tab(20);
 	//char* current_filename=0;
 	mpegOut* current_out;
@@ -304,6 +320,7 @@ int main (int argc,char** argv){
 	programname invocation;
 	bool demux=false;
 	int tmp_seq;
+	FILE *tmp_file=NULL;
 
 
 
@@ -332,7 +349,7 @@ int main (int argc,char** argv){
 	} else {
 		if (!strcmp(progname,"mpgcat"))
 		{
-			invocation=mpgcat;	
+			invocation=mpgcat;
 			joined=true;
 			opresent=true;
 			myoutfile=new char[2];
@@ -350,12 +367,12 @@ int main (int argc,char** argv){
 					want_info=true;
 				}
 				else {
-					if (!strcmp(progname,"mpgdemux")){	
+					if (!strcmp(progname,"mpgdemux")){
 						invocation=mpgdemux;
 						demux=true;
 					}
 					else{
-						if (!strcmp(progname,"tagmp3")){	
+						if (!strcmp(progname,"tagmp3")){
 							return ParseID3Command(argc,argv,1);
 						}
 						else{
@@ -386,31 +403,31 @@ int main (int argc,char** argv){
 		switch(argv[i][0])
 		{
 			case '-':
-				//		this is an option		
+				// this is an option
 				if (lastarg==file && (!want_info)){
 					// an option following a filename
-					// appy range [0-100%]	
-					Tab.ParseRange("]0-]");					
+					// appy range [0-100%]
+					Tab.ParseRange("]0-]");
 				}
-				lastarg=option;	
+				lastarg=option;
 				if((argv[i][1] > '0')&&(argv[i][1]<='9')) {
 					number=&argv[i][1];
 					sscanf(number,"%d",&nparts);
 					if(i+1>=argc){
 						fprintf(stderr,"%s option used without a filename\n",argv[i]);
 						return 1;
-					}	
+					}
 					if(!Tab.AddFile(argv[++i])){
 						fprintf(stderr,"%s is not a valid mpeg file\n",argv[i-1]);
 						if(!want_info) return 1;
-					}				
+					}
 					Tab.Nchunks(nparts);
 					inparts=true;
 				}
 				else {
 					switch (argv[i][1]){
-						case 'd' : 
-							demux=true; 
+						case 'd' :
+							demux=true;
 							break;
 						case 'X' :
 							desperate_mode =true;
@@ -427,117 +444,138 @@ int main (int argc,char** argv){
 							sscanf(&(argv[i][2]),"%d",&tmp_seq);
 							if ((tmp_seq < 0 ) || (tmp_seq > 15))
 							{
- 								fprintf(stderr,"invalid Value for aspect correction %d, exit!\n",tmp_seq);
+								fprintf(stderr,"invalid Value for aspect correction %d, exit!\n",tmp_seq);
 								exit(1);
 							}
 							forced_sequence_header = tmp_seq << 4;
 							break;
 
 						case 'h' : print_help(invocation);
-									  return 0;
-									  break;
+									return 0;
+									break;
 						case 'i' :
-									  want_info=true;
-									  break;
+									want_info=true;
+									break;
 						case 'f' :
-									  confirm_files=false;
-									  break;
+									confirm_files=false;
+									break;
 						case 's' :
-									  inparts=true;
-									  break;
+									inparts=true;
+									break;
 
 						case 'j' :
-									  joined=true;
-									  if(!myoutfile) myoutfile="output.mpg";
-									  break;
+									joined=true;
+									if(!myoutfile) myoutfile="output.mpg";
+									break;
 						case 'T' :
-									  Id3Tag=true;
-									  return ParseID3Command(argc,argv,i+1);
-									  break;
+									Id3Tag=true;
+									return ParseID3Command(argc,argv,i+1);
+									break;
 						case 'b' :
-									  inparts=true;
-									  if (joined) {
-										  fprintf(stderr,"-o and -b options are mutually exclusive\n");
-										  return 1;
-									  } else { if (basename_specified) {
-										  fprintf(stderr,"only one basename is allowed\n");
-										  return 1;				
-									  }
-									  basename_specified=true;
-									  if(i+1>=argc){
-										  fprintf(stderr,"-b option used without basename\n");
-										  return 1;
-									  }
-									  else {
-										  //TODO copy basename
-										  i++;
-										  //						fprintf(stderr,"option -b read, basename : %s\n",argv[i]);
-										  mybasename=new char [strlen(argv[i])+1];
-										  strcpy(mybasename,argv[i]);
-									  }
-									  }
-									  break;
+									inparts=true;
+									if (joined) {
+										fprintf(stderr,"-o and -b options are mutually exclusive\n");
+										return 1;
+									} else { if (basename_specified) {
+										fprintf(stderr,"only one basename is allowed\n");
+										return 1;
+									}
+									basename_specified=true;
+									if(i+1>=argc){
+										fprintf(stderr,"-b option used without basename\n");
+										return 1;
+									}
+									else {
+										//TODO copy basename
+										i++;
+										// fprintf(stderr,"option -b read, basename : %s\n",argv[i]);
+										mybasename=new char [strlen(argv[i])+1];
+										strcpy(mybasename,argv[i]);
+									}
+									}
+									break;
 						case 'o' :
-									  if (basename_specified) {
-										  fprintf(stderr,"-b and -o options are mutually exclusive\n");
-										  return 1;
-									  }
-									  else
-									  {
-										  if (opresent){
-											  fprintf(stderr,"only one -o option allowed\n");
-											  return 1;
-										  }
-										  opresent=true;
-										  if(i+1>=argc){
-											  fprintf(stderr,"-o option used without outfile name\n");
-											  return 1;
-										  }
-										  else {
-											  i++;
-											  //						fprintf(stderr,"option -o read, will join all in file : %s\n",argv[i]);
-											  myoutfile=new char [strlen(argv[i])+1];			
-											  strcpy(myoutfile,argv[i]);
-										  }
-									  }
-									  break;
+									if (basename_specified) {
+										fprintf(stderr,"-b and -o options are mutually exclusive\n");
+										return 1;
+									}
+									else
+									{
+										if (opresent){
+											fprintf(stderr,"only one -o option allowed\n");
+											return 1;
+										}
+										opresent=true;
+										if(i+1>=argc){
+											fprintf(stderr,"-o option used without outfile name\n");
+											return 1;
+										}
+										else {
+											i++;
+											// fprintf(stderr,"option -o read, will join all in file : %s\n",argv[i]);
+											myoutfile=new char [strlen(argv[i])+1];
+											strcpy(myoutfile,argv[i]);
+										}
+									}
+									break;
 						case 'v':
-									  printf("%s Version %s\n",progname,MPGTX_VERSION);
-									  printf("Copyleft 2001 Laurent Alacoque <laureck@users.sourceforge.net>\n");
-									  printf("Partial work 2002 by Philipp Biermann <mpegcut@biermann.org>\n");
-									  printf("updates, bugs, patch, money :    http://mpgtx.sourceforge.net\n");
-									  return 0;
-									  break;
+									printf("%s Version %s\n",progname,MPGTX_VERSION);
+									printf("Copyleft 2001 Laurent Alacoque <laureck@users.sourceforge.net>\n");
+									printf("Partial work 2002 by Philipp Biermann <mpegcut@biermann.org>\n");
+									printf("updates, bugs, patch, money :    http://mpgtx.sourceforge.net\n");
+									return 0;
+									break;
 						case '-':
-									  // this is a long option (--xxxx)
-									  if (strcmp(argv[i],"--force")==0) force_option=true; 
-									  if (strcmp(argv[i],"--no-parachute")==0) catch_sigsegv=false;
-									  break;
+									// this is a long option (--xxxx)
+									if (strcmp(argv[i],"--force")==0) force_option=true;
+									if (strcmp(argv[i],"--no-parachute")==0) catch_sigsegv=false;
+									break;
 						default:
-									  fprintf(stderr,"unrecognized option %s\n",argv[i]);
-									  return 1;
-									  break;
+									fprintf(stderr,"unrecognized option %s\n",argv[i]);
+									return 1;
+									break;
 					}
 				}
 				break;
 
 			case ']':
 			case '[':
-				//		this is a range
-				lastarg=range;
-				if(!Tab.ParseRange(argv[i])){
-					return 1;
+				// this could be a range or a filename beginning with bracket
+				// so check if it is not a file in current dir
+				tmp_file=fopen(argv[i],"r");
+				if(tmp_file==NULL){
+					//	this is a range
+					lastarg=range;
+					if(!Tab.ParseRange(argv[i])){
+						return 1;
+					}
+				}
+				else{
+					//	might be an input file, check!
+					if (lastarg==file && !(want_info)){
+						// two file arguments encountered, apply 100% range to the first
+						// one
+						Tab.ParseRange("]0-]");
+					}
+					if(tmp_file)
+						fclose(tmp_file);
+					lastarg=file;
+					if(!Tab.AddFile(argv[i])){
+						fprintf(stderr,"%s is not a valid mpeg file\n",argv[i]);
+						if(!want_info) return 1;
+					}
 				}
 				break;
+
 			case '{':
-				//      comma separated list of boundaries
+				// comma separated list of boundaries
 				lastarg=range;
 				if(!Tab.ParseBoundaries(argv[i])){
 					return 1;
 				}
 				break;
 			default:
-				//		might be an input file, check!
+				// might be an input file, check!
 				if (lastarg==file && !(want_info)){
 					// two file arguments encountered, apply 100% range to the first
 					//one
@@ -587,15 +625,15 @@ int main (int argc,char** argv){
 	// Handle Info mode
 	/////////////////////////////////////////////////////////////////
 
-	if(want_info){
+	if (want_info){
 		Tab.PrintInfos();
 #ifdef NEED_END_MESSAGE
 		endMessage();
 #endif
 		return 0;
-	}	
+	}
 
-	ChunkTab=Tab.GetChunks(&ChunkCount);	
+	ChunkTab=Tab.GetChunks(&ChunkCount);
 	// from here we need at least one chunk
 	if (ChunkCount==0){
 		fprintf(stderr,"nothing to do \n");
@@ -629,7 +667,7 @@ int main (int argc,char** argv){
 		return 0;
 	}
 
-	//	Tab.PrintTab();
+	// Tab.PrintTab();
 	RTRACK;
 	ATRACK("join mode");
 	/////////////////////////////////////////////////////////////////
@@ -641,7 +679,7 @@ int main (int argc,char** argv){
 			if (ChunkTab[0]->mpegfile->has_video() == false)
 				sprintf(myoutfile, "chunk.mp3");
 			else
-				sprintf(myoutfile, "chunk.mpg");				
+				sprintf(myoutfile, "chunk.mpg");
 		}
 		if (ChunkCount >1) {
 			for ( i = 1; i < ChunkCount ; i++)
@@ -667,7 +705,7 @@ int main (int argc,char** argv){
 		}
 		else {
 			//output to myoutfile
-			//			if (!force_action)
+			// if (!force_action)
 			if (confirm_files && !mpgtx_access(myoutfile))
 			{
 				fprintf(stderr,"file %s already exist. erase ? [n/y/a]:", myoutfile);
@@ -677,17 +715,17 @@ int main (int argc,char** argv){
 				if (answer!='y' && answer !='Y' && answer != 'a' && answer != 'A') {
 					fprintf(stderr,"Aborted\n");
 					return 0;
-				}	
-				if (answer == 'a' || answer == 'A') confirm_files = false;			
+				}
+				if (answer == 'a' || answer == 'A') confirm_files = false;
 			}
 			current_out = TheFac.NewMpegFrom(ChunkTab[0]->mpegfile, myoutfile);
 			if (!current_out)
 				fprintf(stderr,"\nFatal : could not process file %s\n",myoutfile);
 		}
 		if (!current_out){
-			//something went wrong 
+			//something went wrong
 			return 0;
-		}	
+		}
 
 		for ( i=0;i<ChunkCount;i++)
 		{
@@ -745,7 +783,7 @@ int main (int argc,char** argv){
 					sprintf(myoutfile,"%s",mybasename);
 				else
 				{
-					if (ChunkCount==1)	
+					if (ChunkCount==1)
 						sprintf(myoutfile,"%s.mp3",mybasename);
 					else 
 						sprintf(myoutfile,"%s-%0*d.mp3",mybasename,myprec,i+1);
@@ -757,10 +795,10 @@ int main (int argc,char** argv){
 					sprintf(myoutfile,"%s",mybasename);
 				else
 				{
-					if (ChunkCount==1)	
+					if (ChunkCount==1)
 						sprintf(myoutfile,"%s.mpg",mybasename);
 					else 
-						sprintf(myoutfile,"%s-%0*d.mpg",mybasename,myprec,i+1);				
+						sprintf(myoutfile,"%s-%0*d.mpg",mybasename,myprec,i+1);
 				}
 			}	
 			if (confirm_files && !mpgtx_access(myoutfile))
@@ -792,7 +830,7 @@ int main (int argc,char** argv){
 			else
 				fprintf(stderr,"\n");
 			current_out->Finish();
-		}
+		} // for
 		delete [] myoutfile;
 #ifdef NEED_END_MESSAGE
 		endMessage();
@@ -806,7 +844,6 @@ int main (int argc,char** argv){
 	return 0;
 }
 
-
 #ifdef NEED_END_MESSAGE
 void endMessage(void) {
 		fprintf(stderr,"-------Done--------\n");
