@@ -1,51 +1,21 @@
+/*************************************************************
+ *   mpgtx an mpeg toolbox                                   *
+ *   by Laurent Alacoque <laureck@users.sourceforge.net>     *
+ *   (c) 2001                                                *
+ *   You may copy, modify and redistribute this              *
+ *   source file under the terms of the GNU Public License   *
+ ************************************************************/
+
 #include "id3command.hh"
-#include <string.h>
-#include "common.hh"
-
-// needed by move
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <errno.h>
-//for the home dir
-#include <pwd.h>
-// end needed
-
-
-bool id3_Show_Only =false;
-
-typedef struct {
-	char title [31];
-	char artist [31];
-	char album  [31];
-	char year   [5];
-	char comment [31];
-	int track;
-	unsigned char genre;
-
-}myID3tag;
-
-#define NOT_MP3 0
-#define REAL_MP3 1
-#define FAKE_MP3 2
-
-typedef struct {
-	FILE* 		thefile;
-	myID3tag* 	thetag;
-	int			filetype;
-} OpenedFile;
-
-int ParseSet(int argc, char** argv, int argc_offset);
-int ParseShow(int argc, char** argv, int argc_offset);
-int ParseMov(int argc, char** argv, int argc_offset);
-int ParseDel(int argc, char** argv, int argc_offset);
-OpenedFile* OpenFile(char* FileName);
-
-bool SetID3(char* format,OpenedFile* tag);
-void Move2(const char* name,const char* basedir,const char* dest);
-void WriteID3(OpenedFile* filename);
-
 
+/** 
+ * Parses commandline and decide what happens.
+ *
+ * @param argc       argc of calling parent programm
+ * @param argv       argv of calling parent programm
+ * @param arc_offset index begin of interesting args
+ * @return 0 - ok, 1 - error or return code of called subroutine [0;1]
+ */
 int ParseID3Command(int argc, char** argv, int argc_offset){
 	BTRACK;
 	// starting from argc_offset format is :
@@ -62,13 +32,13 @@ int ParseID3Command(int argc, char** argv, int argc_offset){
 
 	if (!strcmp(argv[argc_offset],"-h")){
 		// this is -h option
-		printf("tagmp3 an ID3 tag editor\n");
+		printf("tagmp3 an ID3v1 tag editor\n");
 		printf("Usage : tagmp3 [-n] [-v] [set|move|del|list|show] [FORMAT] [file...]\n\n");
 		printf(" OPTIONS:\n");
 		printf("   -v     prints version information\n");
-		printf("   set    sets ID3 tag according to FORMAT\n");
+		printf("   set    sets ID3v1 tag according to FORMAT\n");
 		printf("   move   moves mp3 file according to the FORMAT\n");
-		printf("   del    removes ID3 tag from file (if any)\n");
+		printf("   del    removes ID3v1 tag from file (if any)\n");
 		printf("   list   lists known music genres along with their numeric values\n\n");
 		printf(" FORMAT:\n");
 		printf("   for set mode   : \"%%Y? %%X:VALUE  ...\" sets the X field of tag\n");
@@ -109,7 +79,7 @@ int ParseID3Command(int argc, char** argv, int argc_offset){
 
 	if (!strcmp(argv[argc_offset],"show")){
 		if (argcount<2){
-			fprintf(stderr,"show id3 tag of what file?\n");
+			fprintf(stderr,"show id3v1 tag of what file?\n");
 			return 1;	
 		}
 		return ParseShow(argc,argv,argc_offset+1);
@@ -117,7 +87,7 @@ int ParseID3Command(int argc, char** argv, int argc_offset){
 
 	if (!strcmp(argv[argc_offset],"set")){
 		if (argcount<3){
-			fprintf(stderr,"Not enough argument for set mode\n");
+			fprintf(stderr,"Not enough arguments for set mode\n");
 			return 1;	
 		}
 		return ParseSet(argc,argv,argc_offset+1);
@@ -133,7 +103,7 @@ int ParseID3Command(int argc, char** argv, int argc_offset){
 
 	if (!strcmp(argv[argc_offset],"del")){
 		if (argcount<2){
-			fprintf(stderr,"remove id3 tag from what?\n");
+			fprintf(stderr,"remove id3v1 tag from what?\n");
 			return 1;	
 		}
 		return ParseDel(argc,argv,argc_offset+1);
@@ -145,6 +115,14 @@ int ParseID3Command(int argc, char** argv, int argc_offset){
 }
 
 
+/** 
+ * Shows ID3v1 TAG of files.
+ *
+ * @param argc       argc of calling parent programm
+ * @param argv       argv of calling parent programm
+ * @param arc_offset index begin of interesting args
+ * @return allways 0, no error if there are no ID3v1 TAGs in file
+ */
 int ParseShow(int argc, char** argv, int argc_offset){
 	BTRACK;
 	OpenedFile* my;
@@ -155,7 +133,7 @@ int ParseShow(int argc, char** argv, int argc_offset){
 		}
 		// okay we have a file here.
 		if (!my->thetag){
-			fprintf(stdout, "%s doesn't have any ID3 tag\n", argv[i]);
+			fprintf(stdout, "%s doesn't have any ID3v1 tag (Note: ID3v2 not supported)\n", argv[i]);
 
 		}
 		else{
@@ -181,17 +159,26 @@ int ParseShow(int argc, char** argv, int argc_offset){
 }
 
 
+/** 
+ * Sets ID3v1 TAG of files.
+ *
+ * @param argc       argc of calling parent programm
+ * @param argv       argv of calling parent programm
+ * @param arc_offset index begin of interesting args
+ * @return allways 0, no error if there are no ID3v1 TAGs in file
+ */
 int ParseSet(int argc, char** argv, int argc_offset){
 	BTRACK;
 	OpenedFile* my;
 	int retval=1;
-		for (int i = argc_offset + 1; i < argc; i++) {
+	for (int i = argc_offset + 1; i < argc; i++) {
 		my = OpenFile(argv[i]);
 		if ((!my->thefile) || (my->filetype != REAL_MP3)) {
 			continue;
 		}
 		// okay we have a file here.
-		if (my->filetype != REAL_MP3) {
+		// FIXME: This could not be reached, DEPRECATED?
+                if (my->filetype != REAL_MP3) {
 			fprintf(stderr, "Skipping file %s ", argv[i]);
 			if (my->filetype== FAKE_MP3) fprintf(stderr,"(Wave file)\n");
 			else fprintf(stderr,"(not an mp3)\n");
@@ -201,18 +188,18 @@ int ParseSet(int argc, char** argv, int argc_offset){
 		if (!SetID3(argv[argc_offset],my)){
 			// something's wrong with the format
 			fprintf(stderr,"ERROR while setting the format\n");
-			return 1;
+			retval=1;
+                        break;
 		}
 		else{
 			if(!id3_Show_Only) {
-				//		printf("Setting %s tag\n",argv[i]);
+				// printf("Setting %s tag\n",argv[i]);
 				WriteID3(my);
 				retval=0;
 			}
 			else{
-				//			printf("\n");
-			}
-
+				// printf("\n");
+		        }
 		}
 
 		// lets delete and close before next opened file
@@ -230,7 +217,14 @@ int ParseSet(int argc, char** argv, int argc_offset){
 
 
 
-
+/** 
+ * Parses command line to prepare moving of MP3 files.
+ *
+ * @param argc       argc of calling parent programm
+ * @param argv       argv of calling parent programm
+ * @param arc_offset index begin of interesting args
+ * @return allways 0, no error if there are no ID3v1 TAGs in file
+ */
 int ParseMov(int argc, char** argv, int argc_offset){
 BTRACK
 	OpenedFile* my;
@@ -387,7 +381,7 @@ BTRACK
 							default : 
 								//invalid format
 								fprintf(stderr,"Invalid format %s\n",&format[i]);
-								return 0;
+								return 1;
 								break;
 						}
 
@@ -452,21 +446,37 @@ BTRACK
 	return 0;
 }
 
-
+/** 
+ * Removes ID3v1 TAG from files.
+ *
+ * @param argc       argc of calling parent programm
+ * @param argv       argv of calling parent programm
+ * @param arc_offset index begin of interesting args
+ * @return 1 - error: windows version dos not work yet, otherwise allways 0
+ */
+int ParseDel(int argc, char** argv, int argc_offset){
+BTRACK
 
 #ifdef _WIN32
 // windows version
-int ParseDel(int argc, char** argv, int argc_offset){
-BTRACK
+/*
+ * from: http://msdn.microsoft.com/library/default.asp?url=/library/en-us/fileio/base/setendoffile.asp
+ * 
+ * The SetEndOfFile function moves the end-of-file (EOF) position for the specified file 
+ * to the current position of the file pointer.
+ *
+ * This function sets the physical end of a file (as indicated by allocated clusters). 
+ * To set the logical end of a file, use the SetFileValidData function.
+ *
+ * BOOL SetEndOfFile(
+ *      HANDLE hFile
+ * );
+ */
 	printf("del\n");
 	fprintf(stderr,"del mode is not implemented under windows : \nstill don't know how to truncate a file");
 	return 1;
-}
 
 #else
-#include <unistd.h>
-int ParseDel(int argc, char** argv, int argc_offset){
-BTRACK
 	off_t FileSize;
 	OpenedFile* my;
 	for (int i=argc_offset; i < argc; i++){
@@ -492,18 +502,24 @@ BTRACK
 					perror(argv[i]);
 				}
 		}
-		//	printf("File doesn't have any tag\n");
+		//printf("File doesn't have any tag\n");
 		// lets delete and close before next opened file
 		if (my->thefile) fclose (my->thefile);
 		if (my->thetag) delete my->thetag;
 		delete my;
 	}
 	return 0;
+        
+#endif // _WIN32
+
 }
-#endif
-
-
 
+/**
+ * Fills a OpenedFile structure with TAG infos of a file according given FileName
+ *
+ * @param  FileName Name of file to open
+ * @return Pointer to filled structure
+ */
 OpenedFile* OpenFile(char* FileName){
 	BTRACK;
 	unsigned char start[4];
@@ -579,6 +595,13 @@ OpenedFile* OpenFile(char* FileName){
 	return my;	
 }
 
+/**
+ * Sets ID3v1 TAG of a MP3 file according given format string.
+ *
+ * @param format Format string with TAGs to set
+ * @param tag    File to tag.
+ * @return false on format errors, true otherwise 
+ */
 bool SetID3(char* format,OpenedFile* tag){
 	BTRACK;
 	// change tag->thetag to reflect format
@@ -647,7 +670,7 @@ bool SetID3(char* format,OpenedFile* tag){
 			// ask user
 			if (fieldsize>0){
 				//any char*
-				fgets(target,fieldsize,stdin);
+				fgets(target,fieldsize+1,stdin);
 				// now remove the trailing \n if any
 				if (strlen(target)!=0){
 					if (target[strlen(target)-1]=='\n') target[strlen(target)-1]=0;
@@ -721,6 +744,11 @@ bool SetID3(char* format,OpenedFile* tag){
 	return true;
 }
 
+/**
+ * Write ID3v1 TAG to given file.
+ *
+ * @param filename MP3 file to set ID3v1 TAG.
+ */
 void WriteID3(OpenedFile* filename){
 BTRACK
 	//write tag->thetag in tag->thefile
@@ -804,19 +832,32 @@ BTRACK
 
 }
 
+/**
+ * Asks user for creation of a new directory.
+ *
+ *@param path Path of new directory
+ *@return true if user allows creation, false if deny
+ */ 
 bool AskDirCreation(char* path){
 BTRACK
-	fprintf(stderr,"Create directory %s ?:",path);
+	fprintf(stderr,"Create directory %s ?:[N/y]",path);
 	char answer=getchar();
 	while(getchar()!='\n');
 RTRACK
-		if (answer=='y' || answer=='Y') return true;
+	if (answer=='y' || answer=='Y') return true;
+	
 	return false;
 }
 
+/**
+ * Moves file to destination, creates intermediate directories if necessary.
+ *
+ * @param name    Filename
+ * @param basedir Source directory
+ * @param dest    Destination directory
+ */
 void Move2(const char* name,const char* basedir,const char* dest){
 BTRACK
-	// move file to  dest, create intermediate dirs if necessary
 	struct stat mystats;
 	int ret;
 	char *tempdest;
