@@ -1,6 +1,6 @@
-/************************************************************
- *   mpgtx an mpeg toolbox                                      *
- *   by Laurent Alacoque <laureck@users.sourceforge.net>     *   
+/*************************************************************
+ *   mpgtx an mpeg toolbox                                   *
+ *   by Laurent Alacoque <laureck@users.sourceforge.net>     *
  *   (c) 2001                                                *
  *   You may copy, modify and redistribute this              *
  *   source file under the terms of the GNU Public License   *
@@ -8,64 +8,77 @@
 
 #include "mpeg.hh"
 #include <stdlib.h>
-//philipp 
+//philipp
 #include <stdio.h>
 extern bool desperate_mode;
 extern bool preserve_header;
+extern bool print_progress;
 int mpeg2found = 0;
 
 // #define _DEBUG_
 int mpeg_bitrate_index [2][3][16] =
-{{{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0},
-	{0,32,48,56,64,80,96,112,128,160,192,224,256,320,384,0},
-	{0,32,40,48,56,64,80,96,112,128,160,192,224,256,320,0}},
-	{{0,32,48,56, 64,80 ,96 ,112,128,144,160,176,192,224,256,0},
-		{0, 8,16,24,32,40,48, 56,64 ,80 ,96 ,112,128,144,160,0},
-		{0, 8,16,24,32,40,48, 56,64 ,80 ,96 ,112,128,144,160,0}}};
+{	// MPEG 1
+	{
+		// layer 1
+		{0,32,64,96,128,160,192,224,256,288,320,352,384,416,448,0},
+		// layer 2
+		{0,32,48,56, 64, 80, 96,112,128,160,192,224,256,320,384,0},
+		// layer 3
+		{0,32,40,48, 56, 64, 80, 96,112,128,160,192,224,256,320,0}
+	},
+	// MPEG 2 or 2.5
+	{
+		// layer 1
+		{0,32,48,56, 64, 80, 96,112,128,144,160,176,192,224,256,0},
+		// layer 2
+		{0, 8,16,24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160,0},
+		// layer 3
+		{0, 8,16,24, 32, 40, 48, 56, 64, 80, 96,112,128,144,160,0}
+	}
+};
 
 int mpeg_sampling_index [3][4] =
 {
 	{44100,48000,32000,0}, //mpeg 1
 	{22050,24000,16000,0}, //mpeg 2
-	{11025,12000,8000,0}   //mpeg 2.5    
-
+	{11025,12000, 8000,0}  //mpeg 2.5
 };
 
-double mpeg_frame_rate_index [9] = 
+double mpeg_frame_rate_index [9] =
 {
-	0., 24000./1001., 24., 25., 
-	30000./1001., 30., 50., 
-	60000./1001., 60. 
+	0., 24000./1001., 24., 25.,
+	30000./1001., 30., 50.,
+	60000./1001., 60.
 };
 
 // ripped from id3ed V1.10.2 (Thx to Matt Mueller)
 // HEY you didn't want me to type *that* !
 const char *genre[MAX_ID3_GENRE]={
-	"Blues",		"Classic Rock",	"Country",			"Dance",			"Disco",			"Funk",
-	"Grunge", 		"Hip-Hop", 		"Jazz", 			"Metal", 			"New Age", 			"Oldies",
-	"Other", 		"Pop", 			"R&B", 				"Rap", 				"Reggae", 			"Rock",
-	"Techno", 		"Industrial", 	"Alternative",		"Ska", 				"Death Metal",		"Pranks",
-	"Soundtrack",	"Euro-Techno",	"Ambient",			"Trip-Hop", 		"Vocal", 			"Jazz+Funk",
-	"Fusion", 		"Trance", 		"Classical",		"Instrumental",		"Acid", 			"House",
-	"Game", 		"Sound Clip", 	"Gospel", 			"Noise", 			"AlternRock",		"Bass",
-	"Soul", 		"Punk", 		"Space", 			"Meditative",		"Instrumental Pop",	"Instrumental Rock",
-	"Ethnic", 		"Gothic", 		"Darkwave", 		"Techno-Industrial","Electronic", 		"Pop-Folk",
-	"Eurodance",	"Dream", 		"Southern Rock",	"Comedy",			"Cult", 			"Gangsta",
-	"Top 40", 		"Christian Rap","Pop/Funk", 		"Jungle", 			"Native American",	"Cabaret",
-	"New Wave", 	"Psychadelic", 	"Rave", 			"Showtunes", 		"Trailer", 			"Lo-Fi",
-	"Tribal", 		"Acid Punk", 	"Acid Jazz", 		"Polka", 			"Retro", 			"Musical",
-	"Rock & Roll",	"Hard Rock", 	"Folk", 			"Folk/Rock", 		"National Folk",	"Swing",
-	"Fast-Fusion",	"Bebob", 		"Latin", 			"Revival", 			"Celtic", 			"Bluegrass", 
-	"Avantgarde",	"Gothic Rock", 	"Progressive Rock",	"Psychedelic Rock", "Symphonic Rock",	"Slow Rock", 
-	"Big Band",		"Chorus", 		"Easy Listening",	"Acoustic", 		"Humour", 			"Speech", 
-	"Chanson",		"Opera", 		"Chamber Music",	"Sonata", 			"Symphony", 		"Booty Bass", 
-	"Primus",		"Porn Groove", 	"Satire", 			"Slow Jam", 		"Club", 			"Tango", 
-	"Samba",		"Folklore", 	"Ballad", 			"Power Ballad",		"Rhythmic Soul", 	"Freestyle", 
-	"Duet",			"Punk Rock", 	"Drum Solo", 		"A capella", 		"Euro-House", 		"Dance Hall",
-	"Goa", 			"Drum & Bass", 	"Club House", 		"Hardcore", 		"Terror",			"Indie", 
-	"BritPop", 		"NegerPunk", 	"Polsk Punk", 		"Beat",				"Christian Gangsta","Heavy Metal", 
-	"Black Metal",	"Crossover", 	"Contemporary C",	"Christian Rock",	"Merengue", 		"Salsa", 
-	"Thrash Metal",	"Anime", 		"JPop",				"SynthPop"
+	"Blues",	"Classic Rock",	"Country",		"Dance",		"Disco",		"Funk",
+	"Grunge", 	"Hip-Hop", 	"Jazz", 		"Metal", 		"New Age", 		"Oldies",
+	"Other", 	"Pop", 		"R&B", 			"Rap", 			"Reggae", 		"Rock",
+	"Techno", 	"Industrial", 	"Alternative",		"Ska", 			"Death Metal",		"Pranks",
+	"Soundtrack",	"Euro-Techno",	"Ambient",		"Trip-Hop", 		"Vocal", 		"Jazz+Funk",
+	"Fusion", 	"Trance", 	"Classical",		"Instrumental",		"Acid", 		"House",
+	"Game", 	"Sound Clip", 	"Gospel", 		"Noise", 		"AlternRock",		"Bass",
+	"Soul", 	"Punk", 	"Space", 		"Meditative",		"Instrumental Pop",	"Instrumental Rock",
+	"Ethnic", 	"Gothic", 	"Darkwave", 		"Techno-Industrial",	"Electronic", 		"Pop-Folk",
+	"Eurodance",	"Dream", 	"Southern Rock",	"Comedy",		"Cult", 		"Gangsta",
+	"Top 40", 	"Christian Rap","Pop/Funk", 		"Jungle", 		"Native American",	"Cabaret",
+	"New Wave", 	"Psychadelic", 	"Rave", 		"Showtunes", 		"Trailer", 		"Lo-Fi",
+	"Tribal", 	"Acid Punk", 	"Acid Jazz", 		"Polka", 		"Retro", 		"Musical",
+	"Rock & Roll",	"Hard Rock", 	"Folk", 		"Folk/Rock",		"National Folk",	"Swing",
+	"Fast-Fusion",	"Bebob", 	"Latin", 		"Revival",		"Celtic",		"Bluegrass",
+	"Avantgarde",	"Gothic Rock", 	"Progressive Rock",	"Psychedelic Rock", 	"Symphonic Rock",	"Slow Rock",
+	"Big Band",	"Chorus", 	"Easy Listening",	"Acoustic", 		"Humour", 		"Speech",
+	"Chanson",	"Opera", 	"Chamber Music",	"Sonata", 		"Symphony", 		"Booty Bass",
+	"Primus",	"Porn Groove", 	"Satire", 		"Slow Jam", 		"Club", 		"Tango",
+	"Samba",	"Folklore", 	"Ballad", 		"Power Ballad",		"Rhythmic Soul", 	"Freestyle",
+	"Duet",		"Punk Rock", 	"Drum Solo", 		"A capella", 		"Euro-House", 		"Dance Hall",
+	"Goa", 		"Drum & Bass", 	"Club House", 		"Hardcore", 		"Terror",		"Indie",
+	"BritPop", 	"NegerPunk", 	"Polsk Punk", 		"Beat",			"Christian Gangsta",	"Heavy Metal",
+	"Black Metal",	"Crossover", 	"Contemporary C",	"Christian Rock",	"Merengue", 		"Salsa",
+	"Thrash Metal",	"Anime", 	"JPop",			"SynthPop"
 };
 
 float mpeg::Duration(){
@@ -110,8 +123,7 @@ bool mpeg::Match(mpeg* peer){
 			return false;
 		}
 
-
-		return true;	
+		return true;
 	}
 
 	if (MpegType == mpeg_VIDEO || MpegType == mpeg_SYSTEM){
@@ -152,7 +164,7 @@ mpeg::mpeg(const char* filename,int verbosity)
 	Audio(0), n_audio(0), Video(0), n_video(0), System(0), Transport(0),
 	start_with_id3(false),
 	mpeg_version(1),
-buffstart(0),buffend(0),buffer(0)
+	buffstart(0),buffend(0),buffer(0)
 {
 	BTRACK;
 
@@ -164,7 +176,8 @@ buffstart(0),buffend(0),buffer(0)
 	// TODO : only on mpeg_VERBOSE ?
 
 	FileName = new char[strlen(filename) + 1];
-	strcpy(FileName, filename);
+	strcpy(FileName
+, filename);
 
 	// open infile read-only, binary type (for compatibility with other than POSIX/GNU)
 	MpegFile = fopen(filename, "rb");
@@ -219,7 +232,7 @@ buffstart(0),buffend(0),buffer(0)
 
 	//	}
 
-	// nothing to do on an empty file	
+	// nothing to do on an empty file
 	if (!FileSize) {
 		if (Verboseness != mpeg_SILENT) {
 			printf("File %s is empty.\n", filename);
@@ -314,11 +327,6 @@ mpeg::~mpeg() {
 
 
 // map a buffer onto file for efficiency
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
 
 byte mpeg::GetByte(off_t offset) {
 	BTRACK;
@@ -326,7 +334,7 @@ byte mpeg::GetByte(off_t offset) {
 	size_t nread;
 	if ((offset >= buffend) || (offset < buffstart))
 	{
-
+		// new buffer need (forward)
 		if (FSEEK(MpegFile, offset, SEEK_SET))
 		{
 			if (Verboseness == mpeg_VERBOSE)
@@ -366,16 +374,12 @@ byte mpeg::Byte(off_t offset){
 
 
 // same as above but improved for backward search
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
 
 byte mpeg::bdGetByte(off_t offset){
 	BTRACK;
 	size_t nread;
 	if ((offset >= buffend) || (offset < buffstart)){
+		// new buffer needed (backward)
 		off_t start = offset-BUFFERSIZE+1 ;
 		start = start >= 0?start:0;
 
@@ -387,7 +391,7 @@ byte mpeg::bdGetByte(off_t offset){
 		if ((offset >= buffend) || (offset < buffstart))
 		{
 			// weird
-			if(Verboseness == mpeg_VERBOSE)
+			if (Verboseness == mpeg_VERBOSE)
 				fprintf(stderr,"could not get offset " _OFF_x " in file %s [" _OFF_x "]\n",
 						offset, FileName, FileSize);
 
@@ -404,12 +408,7 @@ byte mpeg::bdGetByte(off_t offset){
 
 
 // make sure the sequence 0x00 00 01 mark
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
+//
 bool mpeg::EnsureMPEG(off_t offset, marker mark){
 	BTRACK;
 	if ((	GetByte(offset) == 0x00		) &&
@@ -423,15 +422,8 @@ bool mpeg::EnsureMPEG(off_t offset, marker mark){
 
 
 
-
-// get a two byte size
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
-
+// get a two byte size (word)
+//
 unsigned short int mpeg::GetSize(off_t offset ) {
 	BTRACK;
 	return GetByte(offset)*256 + GetByte(offset+1);
@@ -439,18 +431,25 @@ unsigned short int mpeg::GetSize(off_t offset ) {
 }
 
 
+
+// print ID3v2 tag to stdout
+//
 bool mpeg::PrintID3() {
 	BTRACK;
+	// Byte 0-2: ID3v2/file identifier      "ID3"
 	if	(	(GetByte(0)!='I') ||
 			(GetByte(1)!='D') ||
 			(GetByte(2)!='3'))
 		return false;
 
+	// Byte 3-4: ID3v2 version
 	printf("  ID3 v2.%d.%d tag (more info on http://www.id3.org/)\n",
 			GetByte(3), GetByte(4));
 	printf("     ----------------\n");
 
-	// get tag size
+	// Byte 5: ID3v2 flags (%abcd0000)
+
+	// Byte 6-9: ID3v2 tag size (4 * %0xxxxxxx)
 	size_t framesize;
 	off_t tagsize;
 	size_t i;
@@ -471,10 +470,12 @@ bool mpeg::PrintID3() {
 		handled = false;
 		framesize = (GetByte(offset+4)<<24) | (GetByte(offset+5)<<16) |
 			(GetByte(offset+6)<<8) | (GetByte(offset+7));
+		// print tag
 		printf("     %c%c%c%c : ", GetByte(offset), GetByte(offset+1),
 				GetByte(offset+2), GetByte(offset+3));
+
+		// T... - all text tags
 		if (GetByte(offset) == 'T') {
-			//text tag
 			if (!GetByte(offset+10)) //if text encoding = standard encoding
 				for (i = 1; i < framesize; i++) {
 					printf("%c", GetByte(offset + i + 10));
@@ -485,6 +486,7 @@ bool mpeg::PrintID3() {
 				handled=true;
 			}
 		}
+		// COMM - Comments tag
 		if (	(GetByte(offset)  =='C') &&
 				(GetByte(offset+1)=='O') &&
 				(GetByte(offset+2)=='M') &&
@@ -511,27 +513,32 @@ bool mpeg::PrintID3() {
 
 
 
+// check if ID3v2 tag is present and audio follows
+//
 bool mpeg::ParseID3(){
 	BTRACK;
 	if (	(GetByte(0)!='I') ||
-			(GetByte(1)!='D') ||
-			(GetByte(2)!='3'))
+		(GetByte(1)!='D') ||
+		(GetByte(2)!='3'))
 		return false;
 
 	start_with_id3 = true;
+
 	// get tag size
 	size_t tagsize=0;
 	tagsize = GetByte(6)<<21;
 	tagsize |= GetByte(7)<<14;
 	tagsize |= GetByte(8)<<7;
 	tagsize |= GetByte(9);
-	return ParseAudio(tagsize + 10);
 
+	return ParseAudio(tagsize + 10);
 }
 
 
 
+
 // Parses Audio header returns false on not audio
+//
 bool mpeg::ParseAudio(off_t myoffset) {
 	BTRACK;
 	HasAudio = false;
@@ -699,19 +706,12 @@ bool mpeg::ParseAudio(off_t myoffset) {
 	}
 	Audio->duration = ((FileSize*1.0)/Audio->bitrate)*0.008;
 	HasAudio = true;
-
 //	fprintf(stderr, "found audio.\n");
 	return true;
 }
 
 
 
-
-
-
-
-
-
 //Parses Video header returns false on not video
 bool mpeg::ParseVideo(off_t myoffset){
 	BTRACK;
@@ -752,7 +752,7 @@ bool mpeg::ParseVideo(off_t myoffset){
 	//Get Bitrate for duration estimation
 	myoffset += 1; // after picture rate
 
-	// 18 following bytes are the bitrate /400
+	// 18 following bytes are the bitrate /400
 
 	//read first 16 bytes
 	Video->bitrate = GetSize(myoffset);
@@ -764,7 +764,6 @@ bool mpeg::ParseVideo(off_t myoffset){
 
 	//Given bitrate and FileSize, compute an estimation of the duration.
 	Video->duration = (FileSize*8.0) / (Video->bitrate*400);
-
 	//////////// Begin changes 
 	marker mymark;
 	while (true) {
@@ -791,7 +790,7 @@ bool mpeg::ParseVideo(off_t myoffset){
 
 	// find first GOP after video sequence header
 	off_t header_end = FindNextMarker(myoffset, 0xB8);
-	Video->first_gop_offset = header_end;	
+	Video->first_gop_offset = header_end;
 	// did we find header end or are we lost in cyberspace?
 	if (header_end < 0){
 		//could not find first GOP
@@ -819,19 +818,12 @@ bool mpeg::ParseVideo(off_t myoffset){
 		return false;
 	}
 
-	HasVideo = true;		
+	HasVideo = true;
 	return true;
 }
 
 
 
-
-
-
-
-
-
-
 //Parses System header returns false on not system
 bool mpeg::ParseSystem() {
 	BTRACK;
@@ -869,21 +861,23 @@ bool mpeg::ParseSystem() {
 // philipp tries here to search further ....
 			offset +=3;
 			while (found  == 0) {
-				fprintf (stderr,"Searching for a mpeg signature:\n");
-				fprintf(stderr,"Searching for 0x00 at offset:           ");
-				while(GetByte(offset) != 0x00) {
-					fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
+				if (print_progress) {
+					fprintf(stderr,"Searching for a mpeg signature:\n");
+					fprintf(stderr,"Searching for 0x00 at offset:           ");
+				}
+				while (GetByte(offset) != 0x00) {
+					if (print_progress) fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
 					offset++;
 				}
-				fprintf(stderr,"\nSearching for != 0x00 at offset:           ");
-				while(GetByte(offset) == 0x00) {
+				if (print_progress) fprintf(stderr,"\nSearching for != 0x00 at offset:           ");
+				while (GetByte(offset) == 0x00) {
 					offset++;
-					fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
+					if (print_progress) fprintf (stderr,"\b\b\b\b\b\b\b\b\b\b%10qd", offset);
 				}
 				// now from the beginning ..
 				offset -=2;
 				if (!EnsureMPEG(offset, 0xBA)){
-					fprintf(stderr,"\nno success......\n");
+					if (print_progress) fprintf(stderr,"\nno success......\n");
 					offset += 3;
 				}
 				else {
@@ -893,9 +887,8 @@ bool mpeg::ParseSystem() {
 					if (!preserve_header) {
 					fprintf(stderr,"But I will assume this is ok. If you split or join this file, it will be discarded\n");
 					}
-					else
-					{
-					fprintf(stderr,"But I will assume this is ok. And if you split this file, it will be inserted\n");
+					else {
+						fprintf(stderr,"But I will assume this is ok. And if you split this file, it will be inserted\n");
 					}
 					fprintf(stderr,"Before any segment. This movie maybe wil not play with quicktime, use Videolan instead\n");
 					fprintf(stderr,"If you dont want this, please correct the file, so its start with a valid Pack!\n");
@@ -903,8 +896,10 @@ bool mpeg::ParseSystem() {
 				}
 			}
 			}
-			if (found == 0)
+			if (found == 0) {
+				fprintf(stderr,"\nNo success at all.\n");
 				return false;
+			}
 		}
 	}
 
@@ -912,7 +907,7 @@ bool mpeg::ParseSystem() {
 	System = new mpgtx_system;
 	System->audio_system_header = 0;
 	System->video_system_header = 0;
-	System->first_video_packet = 0;	
+	System->first_video_packet = 0;
 
 	// tries to find the two system packets and store them
 	bool keep_going = true;
@@ -943,7 +938,8 @@ bool mpeg::ParseSystem() {
 		// if this is a PACK
 		if (mark==0xBA) {
 			System->muxrate = ReadPACKMuxRate(offset + 4);
-			offset += 12;	//standard pack length
+			//standard pack length
+			offset += 12;
 			continue;
 		}
 
@@ -952,11 +948,11 @@ bool mpeg::ParseSystem() {
 			if (Verboseness == mpeg_VERBOSE)
 
 				fprintf(stderr,
-						"%s: Unhandled packet encountered (%x @" _OFF_x ") while seeking system headers\n",
-						FileName, mark, offset);
+					"%s: Unhandled packet encountered (%x @" _OFF_x ") while seeking system headers\n",
+					FileName, mark, offset);
 
 			offset += 4;
-			continue;						
+			continue;
 		}
 		// it has to be a system packet
 
@@ -972,10 +968,10 @@ bool mpeg::ParseSystem() {
 			//found a PACK before system packet, compute its size (mpeg1 != mpeg2)
 			if ((GetByte(startofpack + 4) & 0xF0) == 0x20) {
 				// standard mpeg1 pack
-				PACKlength=12;		
+				PACKlength=12;
 			}
 			else {
-				if((GetByte(startofpack + 4) & 0xC0) == 0x40){
+				if ((GetByte(startofpack + 4) & 0xC0) == 0x40){
 					// new mpeg2 pack : 14 bytes + stuffing
 					PACKlength = 14 + (GetByte(startofpack + 13) & 0x07);
 				}
@@ -990,19 +986,21 @@ bool mpeg::ParseSystem() {
 		
 		if ((startofpack == -1) || (startofpack + PACKlength != offset)) {
 			// we're probably lost
-				fprintf(stderr,
-						"%s: System Packet not preceded by a PACK [" _OFF_x "] start of pack : " _OFF_x " PACKlength : " _OFF_x " I'll probably crash but I love risk\n",
-						FileName, offset,startofpack,PACKlength);
-				startofpack=offset;
+			fprintf(stderr,
+				"%s: System Packet not preceded by a PACK [" _OFF_x
+				"] start of pack : " _OFF_x " PACKlength : " _OFF_x
+				" I'll probably crash but I love risk\n",
+				FileName, offset,startofpack,PACKlength);
+
+			startofpack=offset;
 
 // keep going anyway			return false;
 		}
-	
 
 
 		//   end of modif.
 
-		ParseSystemPacket(offset,startofpack);		
+		ParseSystemPacket(offset,startofpack);
 
 		packetsize = GetSize(offset + 4);  // to size field
 
@@ -1052,6 +1050,14 @@ bool mpeg::ParseSystem() {
 
 			// keep track of the initial timestamp
 			if (PACKlength == 12) {
+#ifdef _DEBUG_
+				fprintf(stderr,"Now read mpeg initial_TS at [" _OFF_x "] \n",offset - PACKlength);
+				fprintf(stderr,"Should all TS be printed? [n/y]:");
+				if (getchar() == 'y')
+					print_all_ts(0xba);
+				while(getchar() != '\n')
+					;
+#endif
 				System->initial_TS = ReadTS(offset - PACKlength);
 				mpeg2found = 0;
 			}
@@ -1059,6 +1065,7 @@ bool mpeg::ParseSystem() {
 				// off_t firstmpeg2gop = FindNextMarker(1, 0xB8);
 #ifdef _DEBUG_
 				fprintf(stderr,"Now read mpeg2 initial_TS at [" _OFF_x "] \n",offset - PACKlength);
+				fprintf(stderr,"Should all TS be printed? [n/y]:");
 				if (getchar() == 'y')
 				 	print_all_ts(0xba);
 				while(getchar() != '\n')
@@ -1139,18 +1146,21 @@ bool mpeg::ParseSystem() {
 	if (offset != offset2) {
 		// the packet that precedes video sequence is not a PACK
 		offset = start_of_vid_packet;
-	} else {
+	}
+	else {
 		// end May 2
 
 		// March 27 compute pack length
 		if ((GetByte(offset + 4) & 0xF0) == 0x20) {
 			//standard mpeg1 pack
 			PACKlength = 12;
-		} else {
+		}
+		else {
 			if ((GetByte(offset + 4) & 0xC0) == 0x40) {
 				//new mpeg2 pack : 14 bytes + stuffing
 				PACKlength = 14 + (GetByte(offset + 13) & 0x07);
-			} else {
+			}
+			else {
 				//wazup?
 				fprintf(stderr,"Weird PACK encountered\n");
 				PACKlength = 12;
@@ -1200,15 +1210,18 @@ bool mpeg::ParseSystem() {
 
 	//seek the file duration by fetching the last PACK
 	//and reading its timestamp
-
+	// [es] it seems a bad idea on joined MPEGs, where
+	//      timestamps counting began on every cut :-( 
+	
 	off_t last_pack = bdFindNextMarker(FileSize - 13, 0xBA);
 	// -12 because a PACK is at least 12 bytes
 
 	double duration;
 	if ((GetByte(last_pack + 4) & 0xF0) == 0x20){
 		//standard mpeg1 pack
-		duration = ReadTS(last_pack + 4);		
-	} else {
+		duration = ReadTS(last_pack + 4);
+	} 
+	else {
 		last_pack = bdFindNextMarker(FileSize - 8, 0xB8);
 		if(((GetByte(last_pack + 4) & 0xC0) == 0x40) || (mpeg2found == 1)) {
 			//new mpeg2 pack : 14 bytes + stuffing
@@ -1221,7 +1234,8 @@ bool mpeg::ParseSystem() {
 			fprintf(stderr," : %lf\n", duration);
 #endif
 
-		} else {
+		} 
+		else {
 			//wazup?
 			duration = ReadTS(last_pack + 4);
 		}
@@ -1240,14 +1254,6 @@ bool mpeg::ParseSystem() {
 
 
 
-
-
-
-
-
-
-
-
 // print mpeg infos on stdout
 void mpeg::PrintInfos() {
 	BTRACK;
@@ -1258,7 +1264,7 @@ void mpeg::PrintInfos() {
 		if (Audio->mpeg_ver != 3)
 			printf("  Audio : Mpeg %d layer %d\n", Audio->mpeg_ver,Audio->layer);
 		else
-			printf("  Audio : Mpeg 2.5 (rare) layer %d\n", Audio->layer);	
+			printf("  Audio : Mpeg 2.5 (rare) layer %d\n", Audio->layer);
 
 		printf("  Estimated Duration: %s\n", HMS);
 		if (Audio->bitrate) {
@@ -1266,29 +1272,29 @@ void mpeg::PrintInfos() {
 			printf("  Frame size: %d bytes\n", Audio->frame_length);
 		}
 		else {
-			printf("  free bitrate %d Hz (this file is currently unsplitable)\n",
-					Audio->sampling_rate);
+			printf("  free bitrate %d Hz (this file is currently unsplitable)\n",Audio->sampling_rate);
 		}
 		switch (Audio->mode) {
 			case 0: printf("  Stereo,"); break;
 			case 1: printf("  Joint Stereo: "); 
-					  if (Audio->layer==1 || Audio->layer==2) {
-						  switch (Audio->modext) {
-							  case 0: printf("(Intensity stereo on bands 4-31/32)\n"); break;
-							  case 1: printf("(Intensity stereo on bands 8-31/32)\n"); break;
-							  case 2: printf("(Intensity stereo on bands 12-31/32)\n"); break;
-							  case 3: printf("(Intensity stereo on bands 16-31/32)\n"); break;
-						  }
-					  } else {
-						  //mp3
-						  switch (Audio->modext){
-							  case 0: printf("(Intensity stereo off, M/S stereo off)\n"); break;
-							  case 1: printf("(Intensity stereo on, M/S stereo off)\n"); break;
-							  case 2: printf("(Intensity stereo off, M/S stereo on)\n"); break;
-							  case 3: printf("(Intensity stereo on, M/S stereo on)\n"); break;
-						  } 
-					  }
-					  break;
+					if (Audio->layer==1 || Audio->layer==2) {
+						switch (Audio->modext) {
+							case 0: printf("(Intensity stereo on bands 4-31/32)\n"); break;
+							case 1: printf("(Intensity stereo on bands 8-31/32)\n"); break;
+							case 2: printf("(Intensity stereo on bands 12-31/32)\n"); break;
+							case 3: printf("(Intensity stereo on bands 16-31/32)\n"); break;
+						}
+					}
+					else {
+						// mp3
+						switch (Audio->modext){
+							case 0: printf("(Intensity stereo off, M/S stereo off)\n"); break;
+							case 1: printf("(Intensity stereo on, M/S stereo off)\n"); break;
+							case 2: printf("(Intensity stereo off, M/S stereo on)\n"); break;
+							case 3: printf("(Intensity stereo on, M/S stereo on)\n"); break;
+						}
+					}
+					break;
 			case 2: printf("  Dual Channel,"); break;
 			case 3: printf("  Mono,"); break;
 		}
@@ -1305,7 +1311,7 @@ void mpeg::PrintInfos() {
 		else printf("  copy\n");
 
 		if (Audio->tag) {
-			//print id3 tag----------------
+			//print id3v1 tag----------------
 			if (Audio->tag->comment[28] == 0)
 				printf ("  ID3 v1.1 tag\n     ----------------\n");
 			else
@@ -1313,7 +1319,7 @@ void mpeg::PrintInfos() {
 			if (Audio->tag->name[0])
 				printf("     title   : %.30s\n", Audio->tag->name);
 			if (Audio->tag->artist[0])
-				printf("     artist  : %.30s\n", Audio->tag->artist);	
+				printf("     artist  : %.30s\n", Audio->tag->artist);
 			if (Audio->tag->album[0]) {
 				printf("     album   : %.30s\n", Audio->tag->album);
 				if(Audio->tag->comment[28]==0)
@@ -1322,14 +1328,14 @@ void mpeg::PrintInfos() {
 			if (Audio->tag->year[0])
 				printf("     year    : %.4s\n", Audio->tag->year);
 			if (Audio->tag->comment[0])
-				printf("     comment : %.30s\n", Audio->tag->comment);			
+				printf("     comment : %.30s\n", Audio->tag->comment);
 			if (Audio->tag->genre < MAX_ID3_GENRE)
 				printf("     genre   : %s\n", genre[Audio->tag->genre]);
 			printf("     ----------------\n");
 		}	
 
 		if (start_with_id3) PrintID3();
-		printf("\n");		
+		printf("\n");
 		return;
 	}
 
@@ -1379,20 +1385,22 @@ void mpeg::PrintInfos() {
 				Video->bitrate/2500.0);
 		if (UData)
 			printf("\nUser Data:\n------------\n%s------------\n\n",UData->ud);
-		return;	
+		return;
 	}
 
 	if (MpegType == mpeg_SYSTEM ) {
 		if (composite) {
 			printf("  Mpeg System File [%d Video/ %d Audio]\n", n_video,n_audio);
-		} else {
+		} 
+		else {
 			if (HasVideo) {
-				SecsToHMS(HMS, Video->duration);	
+				SecsToHMS(HMS, Video->duration);
 				printf("%s\n", FileName);
 				if (mpeg_version == 1) {
 					printf("  Mpeg 1 System File [Video");
 				}
 				else {
+					// TODO: [es] What is this if-clause for?
 					if (mpeg_version!=2);
 					printf("  Mpeg 2 Program Stream File [Video");
 				}
@@ -1402,6 +1410,20 @@ void mpeg::PrintInfos() {
 				printf("  Muxrate : %.2f Mbps\n", (System->muxrate * 8.0) / 1000000);
 
 				printf("  Estimated Duration: %s\n", HMS);
+
+				if (print_progress) {
+					// do time stamps continuing increase?
+					int tlbreaks = 0;
+					printf("  Checking all time stamps (This may take a while.) ...\n");
+					if ( (tlbreaks = CheckTimeline(0xBA)) ) {
+						printf("  Time line is interupted at %d times.\n",tlbreaks);
+						printf("  That means at these points the time stamp is earlier than the previous one.\n");
+						printf("  You may have problems in joining and splitting this file, sorry.\n");
+					}
+					else
+						printf("  Time line seems to be ok.\n");
+				}
+
 				switch (Video->aspect_ratio) {
 					case 0: printf("  Invalid aspect ratio (forbidden)\n"); break;
 					case 1: printf("  Aspect ratio 1/1 (VGA)\n"); break;
@@ -1443,35 +1465,35 @@ void mpeg::PrintInfos() {
 						Video->frame_rate,
 						(Video->bitrate/2500.0)); 
 			}
-			if(UData )	printf("\nUser Data:\n------------\n%s------------\n\n",UData->ud);			
+			if(UData)	printf("\nUser Data:\n------------\n%s------------\n\n",UData->ud);
 			if (HasAudio){
-				if (Audio->mpeg_ver !=3) printf("  Audio : Mpeg %d layer %d\n",Audio->mpeg_ver,Audio->layer);			
-				else printf("  Audio : Mpeg 2.5 (rare) layer %d\n",Audio->layer);			
+				if (Audio->mpeg_ver !=3) printf("  Audio : Mpeg %d layer %d\n",Audio->mpeg_ver,Audio->layer);
+				else printf("  Audio : Mpeg 2.5 (rare) layer %d\n",Audio->layer);
 				if (Audio->bitrate) printf("  %d kbps  %d Hz\n",Audio->bitrate,Audio->sampling_rate);
 				else printf("  free bitrate %d Hz\n",Audio->sampling_rate);
 
 				switch (Audio->mode){
 					case 0: printf("  Stereo,"); break;
 					case 1: printf("  Joint Stereo: "); 
-							  if (Audio->layer==1 || Audio->layer==2){
-								  switch (Audio->modext){
-									  case 0: printf("(Intensity stereo on bands 4-31/32)\n"); break;
-									  case 1: printf("(Intensity stereo on bands 8-31/32)\n"); break;
-									  case 2: printf("(Intensity stereo on bands 12-31/32)\n"); break;
-									  case 3: printf("(Intensity stereo on bands 16-31/32)\n"); break;															
-								  }
-							  }
-							  else {
-								  //mp3
-								  switch (Audio->modext){
-									  case 0: printf("(Intensity stereo off, M/S stereo off)\n"); break;
-									  case 1: printf("(Intensity stereo on, M/S stereo off)\n"); break;
-									  case 2: printf("(Intensity stereo off, M/S stereo on)\n"); break;
-									  case 3: printf("(Intensity stereo on, M/S stereo on)\n"); break;															
-								  }
+							if (Audio->layer==1 || Audio->layer==2){
+								switch (Audio->modext){
+									case 0: printf("(Intensity stereo on bands 4-31/32)\n"); break;
+									case 1: printf("(Intensity stereo on bands 8-31/32)\n"); break;
+									case 2: printf("(Intensity stereo on bands 12-31/32)\n"); break;
+									case 3: printf("(Intensity stereo on bands 16-31/32)\n"); break;
+								}
+							}
+							else {
+								//mp3
+								switch (Audio->modext){
+									case 0: printf("(Intensity stereo off, M/S stereo off)\n"); break;
+									case 1: printf("(Intensity stereo on, M/S stereo off)\n"); break;
+									case 2: printf("(Intensity stereo off, M/S stereo on)\n"); break;
+									case 3: printf("(Intensity stereo on, M/S stereo on)\n"); break;
+								}
 
-							  }
-							  break;
+							}
+							break;
 					case 2: printf("  Dual Channel,"); break;
 					case 3: printf("  Mono,"); break;
 				}
@@ -1484,7 +1506,7 @@ void mpeg::PrintInfos() {
 				}
 			}
 		}
-		return;		
+		return;
 	}
 
 	if (MpegType== mpeg_TRANSPORT){
@@ -1504,12 +1526,8 @@ void mpeg::PrintInfos() {
 
 
 
-
-
-
-
-
 // convert a time in second to HH:mm:ss notation
+//
 void mpeg::SecsToHMS(char* HMS, float duration){
 	BTRACK;
 	byte hours = (byte)(duration / 3600);
@@ -1529,18 +1547,8 @@ void mpeg::SecsToHMS(char* HMS, float duration){
 
 
 
-
-
-
-
-
 // find next 0x 00 00 01 xx sequence, returns offset or -1 on err
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
+//
 off_t mpeg::FindNextMarker(off_t from){
 	BTRACK;
 	off_t offset;
@@ -1562,16 +1570,9 @@ off_t mpeg::FindNextMarker(off_t from){
 
 
 
-
-
 // find next 0x 00 00 01 xx sequence, returns offset or -1 on err and
 // change mark to xx
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
+//
 off_t mpeg::FindNextMarker(off_t from, marker* mark){
 	BTRACK;
 	off_t offset = FindNextMarker(from);
@@ -1589,18 +1590,8 @@ off_t mpeg::FindNextMarker(off_t from, marker* mark){
 
 
 
-
-
-
-
 // find next 0X00 00 01 mark
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
-
+//
 off_t mpeg::FindNextMarker(off_t from, marker mark){
 	BTRACK;
 	off_t offset = from;
@@ -1625,16 +1616,8 @@ off_t mpeg::FindNextMarker(off_t from, marker mark){
 
 
 
-
-
-
 // same as above but optimized for backward search
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
+//
 off_t mpeg::bdFindNextMarker(off_t from, marker mark) {
 	BTRACK;
 	off_t offset;
@@ -1643,7 +1626,7 @@ off_t mpeg::bdFindNextMarker(off_t from, marker mark) {
 		if(
 			(bdGetByte(offset) == 0x00) &&
 			(bdGetByte(offset+1) == 0x00) &&
-			(bdGetByte(offset+2) == 0x01) && 
+			(bdGetByte(offset+2) == 0x01) &&
 			(bdGetByte(offset+3) == mark))
 		{
 			RTRACK;
@@ -1654,13 +1637,8 @@ off_t mpeg::bdFindNextMarker(off_t from, marker mark) {
 	return -1;
 }
 
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
 
-off_t mpeg::bdFindNextMarker(off_t from, marker* mark) { 
+off_t mpeg::bdFindNextMarker(off_t from, marker* mark) {
 	BTRACK;
 	off_t offset;
 	for (offset = from; offset >= 0; offset--)
@@ -1669,7 +1647,7 @@ off_t mpeg::bdFindNextMarker(off_t from, marker* mark) {
 				(bdGetByte(offset+1)==0x00) &&
 				(bdGetByte(offset+2)==0x01))
 		{
-			*mark=bdGetByte(offset+3);				
+			*mark=bdGetByte(offset+3);
 			RTRACK;
 			return offset;
 		}
@@ -1682,20 +1660,16 @@ off_t mpeg::bdFindNextMarker(off_t from, marker* mark) {
 // read Time Stamp at given offset
 // NOTE: The guys at MPEG are really mad.
 //       look at what we have to do to get a timestamp :/
-
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
-
 double mpeg::ReadTS(off_t offset) {
 	BTRACK;
 	byte highbit;
 	unsigned long low4Bytes;
 	double TS;
+#ifdef _DEBUG_
+//	fprintf(stderr,"Offset: [" _OFF_x "] The bytes...: %2x %2x %2x %2x %2x\n",offset,GetByte(offset + 0),GetByte(offset + 1),GetByte(offset + 2),GetByte(offset + 3),GetByte(offset + 4) );
+#endif
 
-	highbit = (GetByte(offset) >> 3) & 0x01;	
+	highbit = (GetByte(offset) >> 3) & 0x01;
 
 	low4Bytes = ((GetByte(offset) >> 1) & 0x03) << 30;
 	low4Bytes |= GetByte(offset + 1) << 22;
@@ -1710,14 +1684,15 @@ double mpeg::ReadTS(off_t offset) {
 	TS += (double)(low4Bytes);
 	TS /= (double)(STD_SYSTEM_CLOCK_FREQ);
 
+#ifdef _DEBUG_
+//	printf("read TS : %lf, base : 0x%d,%032lx\n",TS,highbit,low4Bytes);
+#endif
+
 	return TS;
 }
 
-#ifndef _WIN32
-#ifndef _MACOSX
-inline
-#endif
-#endif
+
+
 
 double mpeg::ReadTSMpeg2(off_t offset){
 	BTRACK;
@@ -1731,7 +1706,7 @@ double mpeg::ReadTSMpeg2(off_t offset){
 
 	highbit = (GetByte(offset)&0x20)>>5;
 
-	low4Bytes= 		((GetByte(offset) & 0x18) >> 3)	<< 30;
+	low4Bytes = 	((GetByte(offset) & 0x18) >> 3)	<< 30;
 	low4Bytes |= 	 (GetByte(offset) & 0x03) << 28;
 	low4Bytes |=	  GetByte(offset + 1	) << 20;
 	low4Bytes |= 	 (GetByte(offset + 2	) & 0xF8) << 12;
@@ -1751,7 +1726,7 @@ double mpeg::ReadTSMpeg2(off_t offset){
 		// so clockfreq might well be 27MHz / sys_clock_ref
 		//  nonsense: TS /= (double)(27000000 / sys_clock_ref);
 		TS /= (double)(STD_SYSTEM_CLOCK_FREQ);
-	} 
+	}
 
 
 
@@ -1762,6 +1737,9 @@ double mpeg::ReadTSMpeg2(off_t offset){
 	return TS;
 }
 
+
+
+
 off_t mpeg::SkipPacketHeader(off_t myoffset){
 	BTRACK;
 	byte mybyte;
@@ -1791,6 +1769,8 @@ off_t mpeg::SkipPacketHeader(off_t myoffset){
 }
 
 
+
+
 bool mpeg::ParseSystemPacket(off_t startofpacket, off_t startofpack){
 	BTRACK;
 	int size = Read2Bytes(startofpacket + 4);
@@ -1837,6 +1817,8 @@ bool mpeg::ParseSystemPacket(off_t startofpacket, off_t startofpack){
 }
 
 
+
+
 void mpeg::ParseFramesInGOP(off_t offset){
 	BTRACK;
 	int pict_ref;
@@ -1877,13 +1859,14 @@ void mpeg::ParseFramesInGOP(off_t offset){
 			case PaddingPkt: printf("            Padding [" _OFF_x "]\n", off); break;
 
 		}
-		off++;		
+		off++;
 		off=FindNextMarker(off,&mark);
 	}
 }
 
 
 
+
 bool mpeg::ParseExtension(off_t myoffset) {
 	BTRACK;
 	myoffset+=4;
@@ -1895,6 +1878,8 @@ bool mpeg::ParseExtension(off_t myoffset) {
 }
 
 
+
+
 bool mpeg::ParseUserData(off_t myoffset) {
 	BTRACK;
 	off_t end = FindNextMarker(myoffset + 1);
@@ -1939,6 +1924,9 @@ bool mpeg::ParseUserData(off_t myoffset) {
 	return true;
 }
 
+
+
+
 bool mpeg::ParseSequenceExt(off_t myoffset) {
 	BTRACK;
 	unsigned long hsize;
@@ -1973,6 +1961,9 @@ bool mpeg::ParseSequenceExt(off_t myoffset) {
 	return true;
 }
 
+
+
+
 bool mpeg::ParseSequenceDisplayExt(off_t myoffset){
 	BTRACK;
 	if (!DExt) DExt = new display_ext;
@@ -1996,6 +1987,9 @@ bool mpeg::ParseSequenceDisplayExt(off_t myoffset){
 	return true;
 }
 
+
+
+
 bool mpeg::ParseRIFF(){
 	BTRACK;
 	// RIFF
@@ -2027,6 +2021,9 @@ bool mpeg::ParseRIFF(){
 	return false;
 }
 
+
+
+
 off_t mpeg::FindMatchingAudio(off_t myoffset){
 	BTRACK;
 	for( ; myoffset < FileSize - 5; myoffset++) {
@@ -2038,6 +2035,9 @@ off_t mpeg::FindMatchingAudio(off_t myoffset){
 	return -1;
 }
 
+
+
+
 off_t mpeg::bdFindMatchingAudio(off_t myoffset){
 	BTRACK;
 	for( ; myoffset >= Audio->first_frame_offset; myoffset--) {
@@ -2047,6 +2047,9 @@ off_t mpeg::bdFindMatchingAudio(off_t myoffset){
 	return -1;
 }
 
+
+
+
 bool mpeg::MatchAudio(off_t myoffset){
 	BTRACK;
 	// do we have audio anyway?
@@ -2079,6 +2082,9 @@ bool mpeg::MatchAudio(off_t myoffset){
 	return true;
 }
 
+
+
+
 long mpeg::ReadPACKMuxRate(off_t offset) {
 	BTRACK;
 	// at start of a PACK
@@ -2108,15 +2114,6 @@ long mpeg::ReadPACKMuxRate(off_t offset) {
 
 
 
-
-
-
-
-
-
-
-
-
 void mpeg::ParsePAT(off_t offset){
 	BTRACK;
 	//	offset++; //skip the first byte
@@ -2524,7 +2521,6 @@ bool mpeg::ParseTransportStream(off_t offset){
 				Transport->read_pmts++;
 #ifdef _DEBUG_
 				printf("PMT at [" _OFF_x "]\n", payload_offset);
-
 #endif
 
 				break;
@@ -2619,7 +2615,7 @@ transport::transport()
 	PMT_PIDs(0),
 	n_PMT_PIDs(0),
 read_pmts(0)
-{};
+{}
 
 transport::~transport() {
 	BTRACK;
@@ -2653,27 +2649,26 @@ void transport::PrintInfos(){
 		printf("  Mpeg 2 Transport Stream [%d programs]\n",n_progs);
 
 	for(int i = 0; i < n_progs; i++) {
-		printf("    Program N %d contains %d Elementary Streams:\n"
-				,programs[i].prog_num, programs[i].nstreams);
+		printf("    Program N %d contains %d Elementary Streams:\n",programs[i].prog_num, programs[i].nstreams);
 		programs[i].descs.PrintInfos("      ");
 		current=programs[i].TStreams;
 		int stream_count = 1;
 		while (current != 0) {
 			printf("      Stream %d: ", stream_count++);
 			switch (current->type) {
-				case 0: printf("reserved"); break;
-				case 1: printf("MPEG 1 video");break;
-				case 2: printf("MPEG 2 video");break;
-				case 3: printf("MPEG 1 audio");break;
-				case 4: printf("MPEG 2 audio");break;
-				case 5: printf("MPEG 2 private section");break;
-				case 6: printf("MPEG 2 PES with private data");break;
-				case 7: printf("MHEG");break;
-				case 8: printf("DSM_CC");break;
-				case 9: printf("Private data");break;
+				case 0:  printf("reserved"); break;
+				case 1:  printf("MPEG 1 video");break;
+				case 2:  printf("MPEG 2 video");break;
+				case 3:  printf("MPEG 1 audio");break;
+				case 4:  printf("MPEG 2 audio");break;
+				case 5:  printf("MPEG 2 private section");break;
+				case 6:  printf("MPEG 2 PES with private data");break;
+				case 7:  printf("MHEG");break;
+				case 8:  printf("DSM_CC");break;
+				case 9:  printf("Private data");break;
 				default: if (current->type <0x80) printf("MPEG 2 reserved");
-								else printf("User Private data");
-							break;
+						else printf("User Private data");
+					 break;
 			}
 			printf(" [pid: %d]\n", current->pid);
 			current->descs.PrintInfos("        ");
@@ -2724,71 +2719,90 @@ void mpeg_descriptors::PrintInfos(char* prefix){
 	}
 	if (n_unhandled_desc>0) {
 		if (n_unhandled_desc==1)
-			sprintf(temp,"%s1 additional descriptor was not handled\n"
-					,prefix);
+			sprintf(temp,"%s1 additional descriptor was not handled\n",prefix);
 		else
-			sprintf(temp,"%s%d additional descriptors were not handled\n"
-					,prefix, n_unhandled_desc);
+			sprintf(temp,"%s%d additional descriptors were not handled\n",prefix, n_unhandled_desc);
+
 		strcat(description,temp);
 	}
 	printf("%s", description);	
 	delete[] description;
 }
 
-header_buf *readHeader(FILE *myMpegfile, off_t offset, int rw) 
-{
+header_buf *readHeader(FILE *myMpegfile, off_t offset, int rw) {
 
-static byte *nix;
-static off_t size;
-static header_buf *p;
+	static byte *nix;
+	static off_t size;
+	static header_buf *p;
 
-	if (rw == 1)
-	{
+	if (rw == 1) {
 		p = new header_buf;
 		size = offset;
 		nix = new byte[size];
-		if (!nix)
-		{
+		if (!nix) {
 			fprintf(stderr,"unable to alloc buffer for header\n");
 			exit(1);
 		}
-	        FSEEK(myMpegfile,0L,SEEK_SET);		
+		FSEEK(myMpegfile,0L,SEEK_SET);
 		fread(nix,size,1L,myMpegfile);
 		p->size = size;
 		p->buf = nix;
 		if (preserve_header)
 			fprintf(stderr,"Size of Fix: 0x%qx\n",size);
-	return(NULL);
+		return(NULL);
 	}
 	if (rw == 2)
-	{
 		return(p);
-	}
 
-fprintf(stderr,"unable to understand command\n");
-exit(1);
+	fprintf(stderr,"unable to understand command\n");
+	exit(1);
 }
 
 
 
-
+/// prints all time stamps on stdout
 void mpeg::print_all_ts(byte kind){
+	off_t p = 0;
+	byte *nix;
+	double tsx;
+	// FIXME [es] What is "nix" for?
+	nix = &kind;
 
-off_t p = 0;
-byte *nix;
-double tsx;
-nix = &kind;
-
-while (p != -1)
-{
-	p = FindNextMarker(p,kind);
-	tsx = ReadTSMpeg2(p +4);
-	if (p == -1)
-		break;
-	fprintf (stderr,"offset:  " _OFF_x " TS: %lf\n",p,tsx);
-	p+=4;
+	while( (p = FindNextMarker(p,kind)) != -1) {
+		if (mpeg_version == 2)
+			tsx = ReadTSMpeg2(p+4);
+		else
+			tsx = ReadTS(p+4);
 
+		fprintf (stderr,"offset:  " _OFF_x " TS: %lf\n",p,tsx);
+		p+=4;
+	}
 }
 
 
-}
\ No newline at end of file
+
+/// Checks time line of increasing time stamps and return amount of breaks
+int mpeg::CheckTimeline(byte kind){
+	off_t 	p = 0;
+	double 	tsx = 0,
+		tsx_prev = 0;
+	int 	ret = 0;
+
+	while( (p = FindNextMarker(p,kind)) != -1) {
+		if (mpeg_version == 2)
+			tsx = ReadTSMpeg2(p+4);
+		else
+			tsx = ReadTS(p+4);
+
+		if ( !(tsx >= tsx_prev) ){
+			++ret;
+//#ifdef _DEBUG_
+			printf("\tTime stamps jumped from %lf to %lf at position " _OFF_x "\n",tsx_prev,tsx,p);
+//#endif
+		}
+		tsx_prev = tsx;
+		p+=4;
+	}
+	return ret;
+}
+
