@@ -1,6 +1,6 @@
-/************************************************************
- *   mpgtx an mpeg toolbox                                      *
- *   by Laurent Alacoque <laureck@users.sourceforge.net>     *   
+/*************************************************************
+ *   mpgtx an mpeg toolbox                                   *
+ *   by Laurent Alacoque <laureck@users.sourceforge.net>     *
  *   (c) 2001                                                *
  *   You may copy, modify and redistribute this              *
  *   source file under the terms of the GNU Public License   *
@@ -23,6 +23,10 @@ void print_ts(byte *p); // prints the timestamp of a mpeg2 gop - only for testin
 // end philipp mod
 //  #define _DEBUG_
 
+#ifdef _DEBUG_
+int count=0;
+#endif
+
 
 mpegOut::mpegOut(char* filename) {
 	BTRACK;
@@ -1056,13 +1060,12 @@ void mpegOutWithVideo::memWriteTS (long offset, double ts, bool mpeg2pack, bool
 
 	byte uppercode = buffer[offset] & 0xF0;
 	byte hiBit;
-	long lowInt;
+	long lowInt=0;
 	double TS=ts;
 	int hour;
 	int minute;
 	int sec;
 	int pics;
-	double ts9;
 
         if (!mpeg2pack) {
 		;
@@ -1112,10 +1115,10 @@ void mpegOutWithVideo::memWriteTS (long offset, double ts, bool mpeg2pack, bool
 		}
 		else
 		{ // mpeg2 GOP
-			hour = TS / 60 / 60;
-			minute = TS / 60;
-			sec = TS;
-			pics = (TS - ((long) TS)) * 60;
+			hour = int(TS / 60 / 60);
+			minute = int(TS / 60);
+			sec = int(TS);
+			pics = int((TS - ((long) TS)) * 60);
 			buffer[offset] &= 0x7f; // clear hour & 2 bits from minute
 			buffer[offset +1] &= 0x8; // clear minutes & seconds
 			buffer[offset +2] &= 0;
